import femto.mode.HiRes16Color;
import femto.sound.Procedural;

import Common;

class Event
{
    // enum EventType
    // {
    //     NONE, START_BUBBLE, END_BUBBLE
    // }
    
    static final int EVENT_NONE = 0;
    static final int EVENT_START_BUBBLE = 1;
    static final int EVENT_END_BUBBLE = 2;
    static final int EVENT_SFX = 3;
    static final int EVENT_RANDOM_REPEATABLE_SFX = 4;
    
    // enum EventState
    // {
    //     PENDING, RUNNING
    // }
    
    Event() 
    {
        type = EVENT_NONE;
    }
    
    public void clear() 
    {
        eventTime = 0;
        type = EVENT_NONE;
        isDrawingState = false;
    }
    
    public void setBubbleTextEvent(long startTime, String textLinePar1, String textLinePar2, Bubble bubbleImagePar ) 
    {
        long currTime = System.currentTimeMillis();
        eventTime = currTime + startTime;
        type = EVENT_START_BUBBLE;
        isDrawingState = false;  
        bubbleImage = bubbleImagePar;
        textLine1 = textLinePar1;
        textLine2 = textLinePar2;
    }
    
    public void setSfxEvent(long startTime, Procedural sfxPar, float minRandPeriodPar, float maxRandPeriodPar) 
    {
        long currTime = System.currentTimeMillis();
        eventTime = currTime + startTime;
        sfx = sfxPar;
        
        if(minRandPeriodPar>0)
        {
            type = EVENT_RANDOM_REPEATABLE_SFX;
            minRandPeriod = minRandPeriodPar;
            maxRandPeriod = maxRandPeriodPar;
        }
        else
        {
            type = EVENT_SFX;
        }
    }
    
    
    public void execute()
    {
        long currTime = System.currentTimeMillis();
        if( type == EVENT_START_BUBBLE )
        {
            eventTime = currTime + (long)(2*1000);
            type = EVENT_END_BUBBLE;
            isDrawingState = true; 
        }
        else if( type == EVENT_END_BUBBLE )
        {
            clear();    
        }
        else if( type == EVENT_SFX )
        {
            sfx.play();
            clear();      
        }
        else if( type == EVENT_RANDOM_REPEATABLE_SFX )
        {
            // Play sound.
            //System.out.println("Event.execute(). play sfx");
            sfx.play();
            
            // Rattle a new sfx time, and relaunch the event.
            float startTime = ( Math.random() * (maxRandPeriod-minRandPeriod) ) + minRandPeriod;
            setSfxEvent((int)startTime, sfx, minRandPeriod, maxRandPeriod);
        }
    }
    
    public void draw(HiRes16Color screen)
    {
        if( type == EVENT_END_BUBBLE )
        {
            bubbleImage.draw(screen, 100, 60);
            screen.setTextPosition( 100+5, 60+5 );
            screen.textColor = 0;
            screen.print(textLine1);
            screen.setTextPosition( 100+5, 60+5+8 );
            screen.print(textLine2);
         }
    }
    
    public long eventTime;
    public int type;
    public boolean isDrawingState;
    //public EventState state;
    
    // Bubble text event specific parameters
    public String textLine1;
    public String textLine2;
    Bubble bubbleImage;
    
    // Sfx event specific parameters
    Procedural sfx;
    float minRandPeriod; 
    float maxRandPeriod;
}

