// Graphics:
// @hanski, CCBY 3.0
// @vampirics, CCBY 3.0

// Sounds:
// Breviceps, CC0 1.0
// CrocyTC, CC0 1.0
// ArriGD, CC0 1.0
// Missozzy, CCBY 3.0
// Missozzy, CCBY 3.0
// Thanra, CC0 1.0

import femto.mode.HiRes16Color;
import femto.Game;
import femto.State;
import femto.Sprite;
import femto.input.Button;
import ZxSpectrum;
import femto.font.TIC80;
import femto.sound.Mixer;
import femto.sound.Procedural;
//import femto.math;
import femto.palette.Pico8;

import ZombieEntity;
import HorizBarEntity;
import VertBarEntity;
import Common;
import Event;

class Main extends State {
    
    HiRes16Color screen; // the screenmode we want to draw with
    Event[] events;
    HorizBarEntity barH;
    VertBarEntity barV;
    Coffee coffee; // an ansimated sprite imported from Aseprite
    Pattern background; // static image
    Bubble bubbleImage;
    Castle castleImage;
    Zombie zombieWithCoffee;
    arrigd_zombie_roar_3 sfx1;
    breviceps_zombie_gargles sfx2;
    crocytc_zombie3 sfx3;
    missozzy_zombie_02 sfx4;
    missozzy_zombie_04 sfx5;
    thanra_zombie_roar sfx6;
    int currSfxNum;
    float angle; // floats are actually FixedPoint (23.8)
    int counter; // variables are automatically initialized to 0 or null
    int[] isCoffeeTaken;  // Use an int array instead of a boolean array as the latter is not yet supported
    boolean isGameOver;

    // start the game using Main as the initial state
    // and TIC80 as the menu's font
    public static void main(String[] args){
        Game.run( TIC80.font(), new Main() );
    }
    
    // Avoid allocation in a State's constructor.
    // Allocate on init instead.
    void init(){
        
        //
        screen = new HiRes16Color(Pico8.palette(), TIC80.font());
        background = new Pattern();
        bubbleImage =  new Bubble();
        castleImage = new Castle();
        barH = new HorizBarEntity(0, 88-2);
        barV = new VertBarEntity(110-2, 0);
        sfx1 = new arrigd_zombie_roar_3(0);
        sfx2 = new breviceps_zombie_gargles(0);
        sfx3 = new crocytc_zombie3(0);
        sfx4 = new missozzy_zombie_02(0);
        sfx5 = new missozzy_zombie_04(0);
        sfx6 = new thanra_zombie_roar(0);
        currSfxNum = 1;

        // Initialize the Mixer at 8khz
        Mixer.init(8000);
        
        zombieWithCoffee = new Zombie();
        isCoffeeTaken = new int[5];
        
        events = new Event[Common.MAX_EVENTS]; //
        for(int i=0; i < Common.MAX_EVENTS; i++)
        {
            events[i] = new Event();
        }
        
        Common.zombies = new ZombieEntity[Common.MAX_ZOMBIES]; //
        int j=0;
        for(int i=0; i < Common.ZOMBIES_PER_NEST; i++, j++)
        {
            Common.zombies[j] = new ZombieEntity( new Zombie() );
            Common.zombies[j].setX(1*i*5);
            Common.zombies[j].setY(-1*i*5);
            Common.zombies[j].sprite.run();
        }
        for(int i=0; i < Common.ZOMBIES_PER_NEST; i++, j++)
        {
            Common.zombies[j] = new ZombieEntity( new Zombie() );
            Common.zombies[j].setX(-20+220 + (1*i*5));
            Common.zombies[j].setY(-40 +(1*i*5));
            Common.zombies[j].sprite.run();
        }
        for(int i=0; i < Common.ZOMBIES_PER_NEST; i++, j++)
        {
            Common.zombies[j] = new ZombieEntity( new Zombie() );
            Common.zombies[j].setX(-20+220 + (1*i*5));
            Common.zombies[j].setY(40+176+(-1*i*5));
            Common.zombies[j].sprite.run();
        }
        for(int i=0; i < Common.ZOMBIES_PER_NEST; i++, j++)
        {
            Common.zombies[j] = new ZombieEntity( new Zombie() );
            Common.zombies[j].setX(1*i*5);
            Common.zombies[j].setY(176+(1*i*5));
            Common.zombies[j].sprite.run();
        }
       
        coffee = new Coffee();
        coffee.run(); // "run" is one of the animations in the spritesheet
        
        // Test bubble
        events[0].setBubbleTextEvent((long)1*1000, "Zombies are", "coming!", bubbleImage );
        events[1].setBubbleTextEvent((long)5*1000, "They want", "THE COFFEE!", bubbleImage );
        events[2].setBubbleTextEvent((long)10*1000, "How can we", "fight back?", bubbleImage );
        
        // Set events for random sounds.
        events[4].setSfxEvent((long)3*1000, sfx2, 15000, 40000 );
        
    }
    
    // Might help in certain situations
    void shutdown(){
        screen = null;
    }
    
    // update is called by femto.Game every frame
    void update(){
        
       long currTime = System.currentTimeMillis();
        
        // Change to a new state when A is pressed
        if( Button.A.justPressed() )
            Game.changeState( new Main() );
            
        if( Button.B.justPressed() )
        {
            if(currSfxNum==1) sfx1.play();
            else if(currSfxNum==2) sfx2.play();
            else if(currSfxNum==3) sfx3.play();
            else if(currSfxNum==4) sfx4.play();
            else if(currSfxNum==5) sfx5.play();
            else if(currSfxNum==6) sfx6.play();
            
            currSfxNum += 1;
            if( currSfxNum>6 ) currSfxNum = 1;
        }

        // *** Update 
        
        if(!isGameOver)
        {
            // Events
            for(int i=0; i < Common.MAX_EVENTS; i++)
            {
                if(events[i].eventTime > 0 && currTime > events[i].eventTime)
                    events[i].execute();
            }

             for(int i=0; i < Common.MAX_ZOMBIES; i++)
            {
                Common.zombies[i].update();
            }
    
            //
            barH.update();
            barV.update();
            
            //
            checkZombiesCollisionToCastle();
        }
        
        // *** Draw
        
        // Fill the screen using Pattern.png
        for( int y=0; y<176; y += background.height() ){
            for( int x=0; x<220; x += background.width() ){
                background.draw(screen, x, y);
            }
        }

        // Draw  zombies      
        for(int i=0; i < Common.MAX_ZOMBIES; i++)
        {
            Common.zombies[i].draw(screen);
        }
        
        // Draw bars
        barV.draw(screen);
        barH.draw(screen);
        
        // Draw the castle.
        if(!isGameOver)
            castleImage.draw(screen, Common.CASTLE_X, Common.CASTLE_Y);
       
        // Draw coffee or zombie on tower.
        if(isCoffeeTaken[0])
            zombieWithCoffee.draw(screen, 92,         70);
        else
            coffee.draw(screen, 92,         70);
        if(isCoffeeTaken[1])
            zombieWithCoffee.draw(screen, 92+12,      70+10);
        else
            coffee.draw(screen, 92+12,      70+10);
        if(isCoffeeTaken[2])
            zombieWithCoffee.draw(screen, 92+12+10,   70);
        else
            coffee.draw(screen, 92+12+10,   70);
        if(isCoffeeTaken[3])
            zombieWithCoffee.draw(screen, 92,         92);
        else
            coffee.draw(screen, 92,         92);
        if(isCoffeeTaken[4])
            zombieWithCoffee.draw(screen, 92+12+10,   92);
        else
            coffee.draw(screen, 92+12+10,   92);
        
        // Draw events
        for(int i=0; i < Common.MAX_EVENTS; i++)
        {
            if(events[i].eventTime > 0 && events[i].isDrawingState)
                events[i].draw(screen);
        }
        
        // Update the screen with everything that was drawn
        screen.flush();
    }
    

    void checkZombiesCollisionToCastle()
    {
        //sprite.draw(screen); // Animation is updated automatically
        //System.out.println("checkZombiesCollisionToCastle(). START");
        
        // Center point of the bar
        float castX = Common.CASTLE_X;
        float castY = Common.CASTLE_Y;
        float castW = castleImage.width();
        float castH = castleImage.height();
        
        for(int i=0; i < Common.MAX_ZOMBIES; i++)
        {
            ZombieEntity zomb =  Common.zombies[i];
            float zombX = zomb.getX();
            float zombY = zomb.getY();
            float zombW = zomb.sprite.width();
            float zombH = zomb.sprite.height();

            //System.out.println("checkZombiesCollisionToCastle(). i=" + i);
            
            boolean isIntersection = 
                (castX < zombX + zombW) && 
                (castY < zombY + zombH) &&
                (castX + castW > zombX) && 
                (castY + castH > zombY);
                
            // If zombie touches the castle, it will drink one coffee!
            if(isIntersection)
            {
               // Mark the coffee as taken
                for(int j=0; j < 5; j++ )
                    if(!isCoffeeTaken[j])
                    {
                        //System.out.println("checkZombiesCollisionToCastle(). i=" + i + ", j=" + j);
                        isCoffeeTaken[j] = 1;
                        if(j==4) isGameOver = true;
                        break;
                    }
                
                // Put the zombie away    
                zomb.setX(-10);
            }
         }
    }
}
